#include <OneWire.h>                  // Include the oneWire library
#include <DallasTemperature.h>        // Include the DallasTemperatureeWire library
OneWire oneWire(10);                  // Define the pin to which the oneWire bus is connected
DallasTemperature sensors(&oneWire);  // Tell the DallasTemperature librar that the sensors are on the oneWire bus 

// Sensors on breadboard
//uint8_t probe001[8] = { 0x28, 0x72, 0x89, 0x24, 0x03, 0x00, 0x00, 0x4A }; 
//uint8_t probe002[8] = { 0x28, 0x24, 0xA1, 0XB2, 0x03, 0x00, 0x00, 0xF8 };
//uint8_t probe003[8] = { 0x28, 0x8C, 0x4D, 0x24, 0x03, 0x00, 0x00, 0x16 }; 
//uint8_t probe004[8] = { 0x28, 0x23, 0x92, 0xB2, 0x03, 0x00, 0x00, 0xB7 };
//uint8_t probe005[8] = { 0x28, 0xD8, 0xD1, 0xB2, 0x03, 0x00, 0x00, 0x9A }; 
//uint8_t probe006[8] = { 0x28, 0xD1, 0xD7, 0xB2, 0x03, 0x00, 0x00, 0x90 };
//uint8_t probe007[8] = { 0x28, 0x75, 0xA5, 0xB2, 0x03, 0x00, 0x00, 0xFE };
//uint8_t probe008[8] = { 0x28, 0xCF, 0xBA, 0xB2, 0x03, 0x00, 0x00, 0x25 };
//uint8_t probe009[8] = { 0x28, 0xCC, 0xB8, 0xB2, 0x03, 0x00, 0x00, 0xFF };
//uint8_t probe010[8] = { 0x28, 0x5F, 0x6E, 0x24, 0x03, 0x00, 0x00, 0x3D };



//Sensors on Chinese Cable
// create an unsigned 8-bit integer with a probe address
uint8_t probe001[8] = { 0x28, 0xD3, 0x0F, 0xE5, 0x03, 0x00, 0x00, 0xC0 };  
uint8_t probe002[8] = { 0x28, 0x14, 0x07, 0xE5, 0x03, 0x00, 0x00, 0xE4 };  
uint8_t probe003[8] = { 0x28, 0xCE, 0xF8, 0xE4, 0x03, 0x00, 0x00, 0xA9 }; 
uint8_t probe004[8] = { 0x28, 0x24, 0x10, 0xE5, 0x03, 0x00, 0x00, 0x24 };  
uint8_t probe005[8] = { 0x28, 0x24, 0x03, 0xE5, 0x03, 0x00, 0x00, 0x16 };  
uint8_t probe006[8] = { 0x28, 0x30, 0x10, 0xE5, 0x03, 0x00, 0x00, 0xA3 };  
uint8_t probe007[8] = { 0x28, 0x78, 0xFC, 0xE4, 0x03, 0x00, 0x00, 0x03 }; 
uint8_t probe008[8] = { 0x28, 0xA8, 0x0D, 0xE5, 0x03, 0x00, 0x00, 0x23 };  
uint8_t probe009[8] = { 0x28, 0x99, 0xDA, 0xE4, 0x03, 0x00, 0x00, 0x79 };  
uint8_t probe010[8] = { 0x28, 0xA2, 0xE8, 0xE4, 0x03, 0x00, 0x00, 0x6B };

int radioSleepPin = 3;                        // The Arduino pin that is connected to the radio's 'sleep' pin
int radioResetPin = 4;                        // The Arduino pin that is connected to the radio's reset pin
int voltageDividerPin = 0;                    // The Arduino pin that is connected to a point on the voltageDivider that never exceeds +5V
int voltageDividerReading = 0;                // A value for storing our ADC reading
float voltage = 0.0;                          // Varibale for holding our calculated supply voltage
int volts = 0;                                // Variable for holding the voltage '1s'
int tenths = 0;                               // Variable for holding the voltage '0.1s'
float voltageDividerMultiplier = 0.02475;    // Multiplier relative to the particular combination of resistors in our divider
float runTime = 0;

void setup() {
  Serial.begin(1200);                         // Start serial port

  sensors.begin();                            // Start sensors library

  pinMode(radioSleepPin, OUTPUT);             // Pulling this pin HIGH later will put the radio into low-power sleep state
  digitalWrite(radioSleepPin, LOW);           // Set the radio's sleep pin to 0V to ensure it is in non-sleeping state

  pinMode(radioResetPin, OUTPUT);             // Pulling this pin LOW later will reset the radio board
  digitalWrite(radioResetPin, HIGH);          // Set the radio's reset pin to 5V (a negative pulse from here causes a reset)

  delay(150);                                 // At least 150ms delay before resetting radio once after Arduino powers on,
  digitalWrite(radioResetPin, LOW);           // as per advice in radio's datasheet
  delay(5);
  digitalWrite(radioResetPin, HIGH);
}

void loop() {
  wakeRadio();                         // Wake the radio board from it's sleeping state
  for (int i=0; i<10; i++) {            // A loop to obtain and send measurements 10 times with 5 second delay between
    Serial.println("#");
    transmitTemps();
    transmitSupplyVoltage();
    transmitUptime();
    delay(5000);                       // This is the delay between each of the 10 groups of temp readings
  }   
  Serial.println("#");
  sleepRadio();                        // Put the radio back to sleep again
  delay(1200000);                        // delay 30 seconds before repeating the main loop again
// - this will eventually become 1800000 or 20 minutes = 1200000
}

void wakeRadio() {
  digitalWrite(radioSleepPin, LOW);   // Ensure radio's sleep pin is set back to low (wake up)
  delay(100);                          // wait for the radio to reliably wake from sleep state
}

void sleepRadio() {
  digitalWrite(radioSleepPin, HIGH);  // Set radio board's sleep pin high (put radio back to sleep)
}

void transmitTemps() {
  sensors.requestTemperatures();       // Get up to date temperatures from the probes

  Serial.print("1 ");
  printTemperature(probe001);          // Call our sub-routine for obtaining, checking and printing the temp, passing to the sub-routine the
                                       // device-address from the 'probe001' variable defined above
  Serial.print("2 ");
  printTemperature(probe002);
  Serial.print("3 ");
  printTemperature(probe003);
  Serial.print("4 ");
  printTemperature(probe004);
  Serial.print("5 ");
  printTemperature(probe005);
  Serial.print("6 ");
  printTemperature(probe006);
  Serial.print("7 ");
  printTemperature(probe007);
  Serial.print("8 ");
  printTemperature(probe008);
  Serial.print("9 ");
  printTemperature(probe009);
  Serial.print("10 ");
  printTemperature(probe010);
}

void printTemperature(uint8_t deviceAddress[]) {
  float tempC = sensors.getTempC(deviceAddress);  // Put into our float variable 'tempC', the reading from the device whose address matches 'deviceAddress'. deviceAddress was
                                                  // passed to the subroutine when it was called from above - ie, will contain the contents of one of the 'probe0XX' variables
  if (tempC < 0) {tempC = 0;}                     // Check the vallue now contained within variable 'tempC' and if it is invalid (ie less than 0) then set it to 0
  Serial.println(tempC);                          // Print the variable
}

void transmitSupplyVoltage() {
    voltageDividerReading = analogRead(voltageDividerPin);
    voltage = voltageDividerReading * voltageDividerMultiplier;
    volts = (int) voltage;
    tenths = ((voltage * 10) - (volts * 10));
    Serial.print((int) voltage);
    Serial.print(".");    
    Serial.print(tenths);    
    Serial.println(" volts battery");     
}

void transmitUptime() {
  runTime = (millis() / 3600000.0);
  Serial.print(runTime);
  Serial.println(" hours running");
}


